

# Cleaning up environment and loading resources.
#rm (list = ls( ))
source("./util.r")



# SG peer review simulation  _______________________________________________
#
#
replicateSG2013 <- function (
  numberOfAgents = 200,      # Number of agents
  initialAgentResources = "even", # Initial agent resources
  resourcesGain = 1,         # Fixed amount of resources (or 'productivity gain')
  resourcesGainFactor = 1.5,
  acceptanceRate = 0.75,     # or 'publication rate'
  SGbaselineEvaluationBias = 0.1, # or 'b', or 'publication bias by default'
  #authorInvestment = 1, # or "i", see SG:2.14 -  it's implicit in this code
  v = 0.1, # 'velocity of submission quality increase'
  probabilityUnreliableReviewer = 0.5,
  sdSubmissionQuality = 0.1, # it's a constant in SG code
  SGscenario = "no reciprocity",
  gradeLanguage = "none", # or 'random'
  evaluationScale = 5,
  seed = runif(1, 0, 999999999), # random seed
  timeSteps = 10,               # numer of iterations
  printTimeStamp = TRUE,
  exportTimeSeries = FALSE, # If FALSE, the ABM only outputs data pertaining the
                            # the very last simulation step.
  deb = FALSE
) {
  
  # Initialization ('SETUP' in NetLogo)
  #
  #
  set.seed(seed)
  scholars <- createScholars(
    nScholars = numberOfAgents,
    initialScholarResources = initialAgentResources
  )
  #scholars$authorInvestment <- authorInvestment
  
  # Here we initialize some variables.
  scholars$countPublications <- scholars$countReviewsMade <- 
    scholars$countSubmissions <- scholars$discrepancy <- 
    scholars$submissionQuality <- 0
  scholars$deservedPublished <- FALSE
 
  scholars$behavior <- rbinom(numberOfAgents, 1, probabilityUnreliableReviewer)
  for (i in 1:numberOfAgents){ifelse(
    scholars$behavior[i] == 1,
    scholars$behavior[i] <- "unreliable",
    scholars$behavior[i] <- "reliable"
  )}
  
  # If we run the model assuming that agents have different grade languages,
  # we also need to give a set of thresholds to all agents. The grade language is
  # determined by the functioncreateGradeLanguage, which takes as input some
  # parameters determined by the data.frame "criterion".
  # So, first, we create the data.frame criterion. Its content basically says
  # that we want a maximally noisy distribution of thresholds, equivalent to a
  # uniform distribution, and different for each agent.
  criterion <- cbind.data.frame(
      scale = evaluationScale,         # scale (expressed as number of categories)
      gradeLanguage = gradeLanguage # Grade language
    )
  
  # Then we can create the grade language, storing it into a list
  # 'gradeLanguages', where every item is the vector of thresholds for
  # each agent
  if(!is.na(evaluationScale)) {
    gradeLanguages <- createGradeLanguage(
      scholars = scholars,
      criterion = criterion
    )
  }
  
  #if (gradeLanguage == "random"){
  #  criterion <- cbind.data.frame(
  #    scale = evaluationScale,         # scale (expressed as number of categories)
  #    gradeLanguage = c("random"), # Grade language
  #    gradeLanguageHeterogeneity = c(1) # 0(min) to 1(max): degree of variation on a grade language
  #  )
  #  
  #  # Then we can create the grade language, storing it into a list
  #  # 'gradeLanguages', where every item is the vector of thresholds for
  #  # each agent
  #  gradeLanguages <- createGradeLanguage(
  #    scholars = scholars,
  #    criterion = criterion
  #  )
  #}
  
  # We also initialize the object that will contain the results of the 
  # simulation run, and for our later convenience we save the parameters in a
  # dataframe.
  results <- list()
  parameters <- as.data.frame(cbind(
    numberOfAgents,
    initialAgentResources,
    resourcesGain,
    resourcesGainFactor,
    acceptanceRate,
    SGbaselineEvaluationBias,
    v,
    probabilityUnreliableReviewer,
    sdSubmissionQuality, 
    SGscenario,
    gradeLanguage,
    evaluationScale,
    seed,
    timeSteps
  ))
  print(paste0("Timestamp: ", Sys.time(),". Initializing run with:"))
  print(parameters)
  
  
  # The following line implements an assumption in SG:2.14, that half of the
  # scholars have one publication at the start of the simulation.
  scholars$gotPublished <- FALSE
  scholars$countPublications <- 0
  for (i in 1:nrow(scholars)){
    if(rbinom(1,1,0.5)==1){
      scholars$gotPublished[i] <- TRUE
      scholars$countPublications[i] <- 1
    }
  }
  
  
  
  # Simulation (the 'GO' procedure in NetLogo)__________________________________
  #
  #
  # This follows the sequence in the original Netlogo scrips (here combined
  # in one script)
  for (t in 1:timeSteps){
    if(printTimeStamp){print(paste("Time step", t))}
    
    # We update agents' resources, called 'productivity' in SG scripts.
    #scholars$gainByPublication <- SGcalculateGainByPublication(scholars)
    scholars$resources <- updateScholarsResources(
      scholars=scholars,
      resourcesGain = resourcesGain,
      resourcesGainFactor = resourcesGainFactor,
      acceptanceRate = acceptanceRate
    )
    
    # Defining which role agents are going to take in this round, author or rev.
    scholars$role <- NA
    scholars$role[sample(1:numberOfAgents, numberOfAgents/2)] <- "author"
    scholars$role[is.na(scholars$role)] <- "reviewer"
    
    # Then we calculate the expected quality
    scholars$expectedQuality <- sg2013expectedQuality(v,scholars$resources)
    
    # For our convenience, we save the index of agents who are playing authors
    # and reviewers in this round.
    authors <- scholars$id[scholars$role=="author"]
    reviewers <- scholars$id[scholars$role!="author"]
    
    # Matching authors and reviewers (random matching).
    scholars$nw <- allocation(
      agentsetA = authors,
      agentsetB = reviewers,
      rule = "random"
    )

    # Setting reviewers' behavior.
    # This depends on the parameter SGscenario, which allows to switch between
    # the scenarios in SG
    scholars$behavior <- sg2013scholarsBehavior(
      scholars, reviewers, SGscenario, probabilityUnreliableReviewer
    )
    
    # Preparing submissons
    # In this context, this means looping through the authors, and for each of
    # them calculating (1) the intrinsic quality of their submission, and (2) 
    # the cost of preparing a submission.
    submissions <- SGprepareSubmission(
      scholars = scholars,
      authors = authors,
      SGscenario = SGscenario
    )
    scholars$submissionQuality <- submissions[,1]
    scholars$costOfAuthoring <- submissions[,2]
    scholars[authors,]$countSubmissions <- scholars[authors,]$countSubmissions+1
    
    # Calculating costs of reviewing.
    scholars$costOfReviewing <- 0
    for (i in reviewers){
      scholars$costOfReviewing[i] <- sgCalculateCostOfReviewing(
        reviewerResources = scholars$resources[i],
        reviewerExpectedQuality = scholars$expectedQuality[i],
        submissionQuality = scholars$submissionQuality[scholars$nw[i]],
        reviewerBehavior = scholars$behavior[i]
      )
      scholars$countReviewsMade[i] <- scholars$countReviewsMade[i] + 1
    }
    
    # Reviewing
    # Here, reviewing means producing a score to estimate a submission's quality.
    scholars$reviewerAssessment <- NA
    for (i in authors){
      scholars$reviewerAssessment[i] <- rate(
        evaluatedAttribute = scholars$resources[i],
        rule="qualitySG2013",
        reviewerBehavior = scholars$behavior[scholars$nw[i]],
        SGbaselineEvaluationBias = SGbaselineEvaluationBias,
        SGvelocity = 0.1
      )
      
      # If we are running the model assuming reviewers have grade languages,
      # then the reviewerAssessment needs to be discretized according to the
      # reviewer's grade language:
      #if (gradeLanguage == "random"){
      if (!is.na(evaluationScale)){
        scholars$reviewerAssessment[i] <- SG2013discretizeWithGradeLanguage(
          score = scholars$reviewerAssessment[i],
          thresholds = gradeLanguages[[scholars$nw[i]]],
          scale = criterion$scale
        )
      }
    }
    
    # Calculating the accuracy of the review
    # The accuracy, here called 'discrepancy', is merely the absolute difference
    # between the true quality of the submission and the reviewer's score that
    # tried to estimate it. Larger discrepancy means a less accurate review.
    qualitySelfPerception <- scholars$submissionQuality
    
    # Note that if we are running the model with random grade languages, the
    # authors' self-perception of their work quaity needs to be discretized 
    # according to the authors' grade language:
    #if (gradeLanguage=="random"){
    if (!is.na(evaluationScale)){
      for (i in authors){
        qualitySelfPerception[i] <- SG2013discretizeWithGradeLanguage(
          score = qualitySelfPerception[i],
          thresholds = gradeLanguages[[i]],
          scale = criterion$scale
        )
      }
    }
    
    scholars[authors,]$discrepancy <-
      abs(qualitySelfPerception[authors] - 
            scholars[authors,]$reviewerAssessment)
    
    # Selecting publications
    # Here we determine which submissions get published. To do so, we order them
    # by the score they received from the reviewers, we locate a threshold based
    # on the acceptance rate (which is a model parameter), and flag as 
    # 'published' all those whose score makes it past the threshold.
    # At the same time and in the same way, we determine which submissions would
    # be published if the review process was maximally effective.
    scholars[authors,]$gotPublished <- FALSE
    scholars[authors,]$deservedPublished <- FALSE
    
    nSubmissionsToBePublished <- round(length(authors) * acceptanceRate)
    
    ranking <- sort(
      scholars[scholars$role == "author",]$reviewerAssessment,
      decreasing = TRUE,
      index.return=TRUE
    )
    
    idealRanking <- sort(
      scholars[scholars$role == "author",]$submissionQuality,
      decreasing = TRUE,
      index.return=TRUE
    )
    
    for(i in 1:nSubmissionsToBePublished){
      scholars[authors[ranking[[2]][i]],]$gotPublished <- TRUE
      scholars[authors[ranking[[2]][i]],]$countPublications <-
        scholars[authors[ranking[[2]][i]],]$countPublications + 1
      scholars[authors[idealRanking[[2]][i]],]$deservedPublished <- TRUE
    }
    
    if(exportTimeSeries){
      globalVariables <- SG2013calculateGlobalVariables(scholars, parameters)
      results[[t]] <- list (globalVariables, scholars)
    }
  }
  
  if (exportTimeSeries == FALSE){
    globalVariables <- SG2013calculateGlobalVariables(scholars, parameters)
    results <- list (globalVariables, scholars)
  }
  
  if(deb){
    d <<- list(
      scholars = scholars,
      submissions = submissions,
      gradeLanguages = gradeLanguages,
      authors = authors,
      reviewers = reviewers
    )
  }
  
  return(results)
}


if (FALSE){
# This launches the ABM:
  results <- replicateSG2013(
  numberOfAgents = 10,
  acceptanceRate = 0.25,
  timeSteps = 200,
  exportTimeSeries = TRUE,
  #SGscenario = "no reciprocity"
  #SGscenario="indirect reciprocity"
  #SGscenario="fairness"
  #SGscenario = "self-interested authors"
  SGscenario = "fair authors"
)
}