# This script contains basic functions that regulate agents' behavior.
#

# Initialization functions _____________________________________________________



createScholars <- function(
  nScholars,
  initialScholarResources = "even"
){
  id <- c(1:nScholars)
  scholars <- data.frame(id)
  if (exists("institutions")){
    scholars$institution <- sample(institutions$id, nScholars, replace=TRUE)
  }
  resources <- c()
  ifelse(
    initialScholarResources == "even",
    resources <- rep.int(0, nScholars),
    resources <- sample(0:10, size=nScholars, replace=TRUE)
  )
  scholars$resources <- resources
  return(scholars)
}



# A grade language is a vector of thresholds used to discretize a continuous 
# quantity (e.g. a rate).
# This function takes as input a criterion and the scholars data.frame, and for
# each it returns a list where each item is the grade language of a scholar.
createGradeLanguage <- function(
  scholars = reviewers,
  criterion = criteria[1,] # A row of the criteria dataframe, the 1st by default
)
{
  gradeLanguages <- list()
  #thresholds <- c()
  
  # If the grade language is random, then we create a grade language where the
  # thresholds are drawn from a uniform distribution.
  #if (criterion$gradeLanguage == "random"){
  #  thresholds <- runif (
  #    n = criterion$scale - 1, # This determines how many thresholds we need
  #    min = 0, max = 1
  #  )
  #  thresholds <- thresholds[order(thresholds)]
  #}
  #
  ## If the grade language is symmetric, then the thresholds are evenly spread
  ## in the range.
  #if (criterion$gradeLanguage == "symmetric"){
  #  thresholds <- (1:criterion$scale - 1) / criterion$scale
  #  thresholds <- thresholds[-1]#[c(-1, -criterion$scale)]
  #}
  if (criterion$gradeLanguage == "none") {
    for (i in 1:nrow(scholars)){
      thresholds <- (1:criterion$scale - 1) / criterion$scale
      gradeLanguages[[i]] <- thresholds[-1]
    }
  }
  if (criterion$gradeLanguage == "random") {
    for (i in 1:nrow(scholars)){
      thresholds <- runif (
        n = criterion$scale - 1, # This determines how many thresholds we need
        min = 0, max = 1
      )
      gradeLanguages[[i]] <- thresholds[order(thresholds)]
    }
  }
  
  # Then, for each agent, we determine how much noise (if any)
  # there needs to be around those thresholds
  #for (i in 1:nrow(scholars)){
  #  if (criterion$gradeLanguageHeterogeneity == 0) {
  #    gradeLanguages[[i]] <- thresholds
  #  } else {
  #    iThresholds <- thresholds
  #    for (t in 1:length(thresholds)){
  #      minT <- iThresholds[t] - criterion$gradeLanguageHeterogeneity
  #      if (minT < 0) {minT <- 0}
  #      maxT <-iThresholds[t] + criterion$gradeLanguageHeterogeneity
  #      if(maxT > 1) {maxT <- 1}
  #      iThresholds[t] <- runif(n=1, min= minT, max= maxT)
  #    }
  #    gradeLanguages[[i]] <- iThresholds[order(iThresholds)]
  #  }
  #}
  
  return(gradeLanguages)
}


SG2013discretizeWithGradeLanguage <- function(score,thresholds,scale){
  if (score < 0 | score > 1){
    score <- truncate(score)
    #warning("Score out of bounds")
  }
  return(findInterval(score, thresholds)/(scale -1))
}


truncate <- function(x, min = 0, max = 1){
  if (length(x) > 1) {return( sapply(x, truncate, min = min, max = max))}
  ifelse(
    x < min,
    return(min),
    ifelse(
      x > max,
      return(max),
      return(x)
    )
  )
}


updateScholarsResources <- function ( # This replicates SG
  scholars,
  resourcesGain = 1,
  resourcesGainFactor = 1.5,
  acceptanceRate = 0.5
  ){
  # sanitizing input:
  if(is.null(scholars$costOfAuthoring)){scholars$costOfAuthoring <- 0}
  if(is.null(scholars$costOfReviewing)){scholars$costOfReviewing <- 0}
  
  # in the code by SG, this function corresponds to their
  # 'update-productivity' procedure.
  gainByPublication <- sg2013calculateGainByPublication(
    scholars,
    resourcesGainFactor,
    acceptanceRate
  )
  resources <- 
    scholars$resources +
    resourcesGain +
    gainByPublication -
    scholars$costOfAuthoring -
    scholars$costOfReviewing
  return(resources)
}

sg2013expectedQuality <- function (v,res){ #This is SG:eq.1
  return((v * res) / (v * res + 1))
}


# Here we implement the reviewer behavior form SG - specifically, this
# function determines whether a reviewer will behave reliably (giving accurate
# scores) on unreliably (giving low quality scores).
# The first three scenarios from SG are modeled here.
sg2013scholarsBehavior <- function(scholars, reviewers, SGscenario, p){
  x <- rep(NA, nrow(scholars))
  
  # ----------------------------------------------------------------------------
  # 1) No reciprocity. Reviewer behavior will be random.
  if (SGscenario == "no reciprocity"){ 
    for (i in reviewers){x[i] <- "reliable"}
    
    # find unreliable reviewers
    u <- sample(reviewers, size = round(p * length(reviewers)))
    for (i in u){x[i] <- "unreliable"}
  }
  
  # ----------------------------------------------------------------------------
  # 2) Indirect reciprocity. Reviewers will behave reliably if their own 
  #    previous submission was published. Otherwise, they will be unreliable.
  if (
    SGscenario == "indirect reciprocity" | 
    SGscenario == "self-interested authors" # > In SG, this author strategy,
  ){                                        # self-interested authors, is always
    for (i in reviewers){                   # assumed incombination with indirect
      ifelse(                               # reviewer reciprocity.
        scholars$gotPublished[i] == TRUE,
        x[i] <- "reliable",
        x[i] <- "unreliable"
      )
    }
  }
  
  # ----------------------------------------------------------------------------
  # 3) Fairness. Reviewers will be reliable if their own previous submission as
  #    authors was evaluated properly. Unreliable otherwise.
  if (
    SGscenario == "fairness" |
    SGscenario == "fair authors"  # As above, in SG author fair behavior is
  ){                              # only explored in combination with reviewer
    for (i in reviewers){ 
      # fair behavior.
      
      if(scholars$discrepancy[i] == 0 | is.na(scholars$discrepancy[i])) {
        x[i] <- "reliable"
      }else{
       ifelse(
         scholars$discrepancy[i] < 0.1 * scholars$submissionQuality[i], # 0.1 comes from SG:3.3
         x[i] <- "reliable",
         x[i] <- "unreliable"
         )
      }
    }
  }
  return(x)
}


# This function replicates how SG calculate the quality of a submission.
# and the costs (in resources) that the author incurs.
# It also implements two author strategies, 'self-interested authors' and
# 'fair authors'.
SGprepareSubmission <- function(
  scholars,
  authors,
  SGscenario,
  sdSubmissionQuality = 0.1,
  authorSpentFactorForArticle = 1
  # the latter represents the variable author-spent-factor-for-article from the
  # original scripts SG. Since it seems an innocent paramenter and they
  # do not manipulate it either, for now I'd leave it implicit, meaning that
  # it's hard coded and not a model parameter - for now at least.
){
  submissionQuality <- scholars$submissionQuality#rep(NA, times = nrow(scholars))
  costOfAuthoring <- rep(0, times = nrow(scholars))
  
  # ----------------------------------------------------------------------------
  # No author strategy. Authors always put max effort into (and pay max price
  # for) preparing a submission.
  if (
    SGscenario != "self-interested authors" &
    SGscenario != "fair authors"
  ){
    for (i in authors){
      #print(i)
      submissionQuality[i] <- rnorm(
        1,
        mean = scholars$expectedQuality[i],
        sd = abs(scholars$expectedQuality[i] * sdSubmissionQuality)
      )
      costOfAuthoring[i] <- 
        scholars$resources[i] * authorSpentFactorForArticle
    }
  }
  
  # ----------------------------------------------------------------------------
  # 4) Self-interested authors. Authors will put max effort (and pay max price)
  #    only if their previous submission was published. Little effort and low
  #    costs otherwise.
  if (SGscenario == "self-interested authors"){
    for (i in authors){
      if(
        scholars$gotPublished[i] == TRUE |
        scholars$countSubmissions[i] == 0
      ) {
        submissionQuality[i] <- rnorm(
          1,
          mean = scholars$expectedQuality[i],
          sd = abs(scholars$expectedQuality[i] * sdSubmissionQuality)
        )
        costOfAuthoring[i] <- 
          scholars$resources[i] * authorSpentFactorForArticle
      } else {
        submissionQuality[i] <- rnorm(
          1,
          mean = scholars$expectedQuality[i] * 0.1,
          sd = abs(scholars$expectedQuality[i] * sdSubmissionQuality)
        )
        costOfAuthoring[i] <- 
          scholars$resources[i] * authorSpentFactorForArticle * 0.1
      }
    }
  }
  
  # ----------------------------------------------------------------------------
  # 5) Fair autors. Authors put max effort and pay max price only if their
  #    previous submissions were evaluated accurately enough (that is, if the
  #    'discrepancy' was not too wide). Else, they will use fewer resources and
  #    pay a lower price.
  if (SGscenario == "fair authors"){
    for (i in authors){
      
      # If last time the author got a fair review (or was never reviewed before)
      if(
        scholars$discrepancy[i] < 0.1 * scholars$submissionQuality[i] |
        scholars$countSubmissions[i] == 0
      ){
 
        # Then produce a good quality submission
        submissionQuality[i] <- rnorm(
          1,
          mean = scholars$expectedQuality[i],
          sd = abs(scholars$expectedQuality[i] * sdSubmissionQuality)
        )
        costOfAuthoring[i] <- 
          scholars$resources[i] * authorSpentFactorForArticle
      
      } else {
        
        # Else, produce a low quality submission
        submissionQuality[i] <- rnorm(
          1,
          mean = scholars$expectedQuality[i] * 0.1,
          sd = abs(scholars$expectedQuality[i] * sdSubmissionQuality)
        )
        costOfAuthoring[i] <- 
          scholars$resources[i] * authorSpentFactorForArticle * 0.1
      }
    }
  }
  
  # Submission quality and cost are returned as two columns of a matrix:
  return(cbind(submissionQuality,costOfAuthoring))
}


# In SG, scholars who get published see their resources multiplied by
# a factor between 1 and 1.5, depending on their productivity/resources, where
# agents with more resources have a lower multiplier, and vice-versa
# (see SG:2.6, 2.14). How exactly the multuplier is calculated is not
# explained in the paper. Thus, here we make an assumption.
sg2013calculateGainByPublication <- function(
  scholars,
  resourcesGainFactor = 1.5,
  acceptanceRate = 0.5
){
  countPublications <- #round(
    nrow(scholars[scholars$role=="author",]) * acceptanceRate#)
  
  # Ordering agents by productivity (which symbolizes career stage)
  orderedScholars <- scholars[order(scholars$resources),]
  orderedAuthors <- subset(
    orderedScholars,
    orderedScholars$role == "author" & orderedScholars$gotPublished == TRUE
  )
  
  gainVector <- rep (0, nrow(scholars))
  limiter <- counter <- 0
  for (i in 1:nrow(orderedAuthors)){
    limiter <- (counter / (countPublications - 1)) * (resourcesGainFactor - 1)
    gainByPublication <- 
      orderedAuthors$resources[i] * (resourcesGainFactor - limiter)
    #print(gainByPublication)
    gainVector[orderedAuthors$id[i]] <- gainByPublication
    counter <- counter + 1
  }
  
  return(gainVector)
}


# Allocation rules______________________________________________________________
# 
# Here we create a network of reviewers and authors/applicants/submissions
allocation <- function(
  agentsetA,
  agentsetB,
  rule = "random"
){
  n = length(agentsetA) + length(agentsetB)
  if(rule=="random"){
    # In this case, the network we produce is in the form of a nodelist.
    x <- c()
    agentsetA <- agentsetA[sample(1:length(agentsetA))]
    for(i in 1:n){
      edge <- c(agentsetA[i], agentsetB[i])
      x[edge[1]] <- edge[2]
      x[edge[2]] <- edge[1]
    }
    return(x)
  }
}


# How a referee reviews a submission ___________________________________________
#
# This function takes as input a submission, and returns a referee's 
# assessment. For now we have three ways in which a reviewer produces an
# evaluation: random, based on quality, or replicating SG.
rate <- function(
  evaluatedAttribute,
  rule = "quality",
  sd = 0,
  categories = 2,
  probWeights = NA,
  thresholds = NA, 
  reviewerBehavior = "reliable",
  SGbaselineEvaluationBias = 0.1,
  SGvelocity = 0.1 #SG:table 1
)
{
  # Checking input for validity
  if(categories < 2 | categories != round(categories)){
    warning("Invalid number of categories fed to the function *rate*.")
  }
  
  # Random rating:
  # This rule returns one of the possible values of the evaluation scale
  # (i.e. integers from 1 to the number of categories), drawn at random from
  # a uniform distribution. If specified, probability weights can be assigned
  # for each of the categories.
  if (is.na(probWeights)) {probWeights <- rep(1/categories, categories)}
  if (rule=="random"){
    return(sample(1:categories, size=1, prob=probWeights)-1)#With 2 categories: 1=accept, 0=reject
  }
  
  # Quality-based rating:
  # A raw score in the range [0,1] is produced by estimating the true quality.
  # The estimation is drawn from a normal distribution, with mean equal to the
  # 'quality' of the evaluated attribute, and s.d. = a given amount of error.
  # Then, the raw score is transformed to the approrpiate evaluation scale,
  # which is essentially a discretization of the range [0,1]. By default, the
  # scale is discretized at regular intervals; however, in case thresholds are
  # provided, these are used as a "grade language" and used for discretizing.
  if (rule=="quality"){
    rawScore <- truncate(rnorm(1, mean=evaluatedAttribute, sd=sd))
    ifelse(
      is.na(thresholds),
      return(round((categories - 1) * rawScore, digits = 0)),
      return(findInterval(rawScore, thresholds))
    )
  }
  
  # SG rating:
  # As in the root model, here the rating is on a countinuous scale and is based
  # on the reviewer's approximation of the amount of resources at the author's
  # disposal (a proxy for the 'true quality').
  if (rule=="qualitySG2013"){
    b <- SGbaselineEvaluationBias # SG:11, SG:table 1 -  parameter 'evaluation bias by default'
    overratingFactor <- 1 + 90 / 100 # This is parameter o in SG:table 1
    underratingFactor <- 1 - 90 / 100 # This is parameter u in SG:table 1
    
    # Now, the following code implements the evaluation bias as in 2013:2.11.
    # The following code literally copies what the SG NetLogo code does.
    if (reviewerBehavior == "reliable"){ #SG:2.10
      guessedAuthorResources <- rnorm(
        n=1,
        mean=evaluatedAttribute,
        sd=abs(evaluatedAttribute * b)
      )
    }
    if(reviewerBehavior=="unreliable"){ #SG:2.12
      ifelse(
        rbinom(1,1,0.5)==1, # Bernoulli trial
        guessedAuthorResources <- rnorm(
          n=1,
          mean=evaluatedAttribute * overratingFactor,
          sd=abs(evaluatedAttribute * overratingFactor * b)
        ),
        guessedAuthorResources <- rnorm(
          n=1,
          mean=evaluatedAttribute * underratingFactor,
          sd=abs(evaluatedAttribute * underratingFactor * b)
        )
      )
      if(guessedAuthorResources < 0){guessedAuthorResources <- 0}
    }
    return(
      (SGvelocity*guessedAuthorResources)/(SGvelocity*guessedAuthorResources +1)
    )
  }
}


# In the root model SG, reviewers pay a cost for reviewing. Here is how it
# is calculated
sgCalculateCostOfReviewing <- function( # From SG:2.8 (equation 2)
  reviewerResources,
  reviewerExpectedQuality,
  submissionQuality,
  reviewerBehavior
){
  ifelse(
    reviewerBehavior == "reliable",
    return(
      0.5 * reviewerResources * (1+ (submissionQuality-reviewerExpectedQuality))
    ),
    return(
      0.25 * reviewerResources * (1+ (submissionQuality-reviewerExpectedQuality))
    )
  )
}


# Functions to calculate indices _______________________________________________

# This function calculates the Gini coefficient.
giniIndex <- function (x, isSample = FALSE) 
{
  x <- sort(as.numeric(x[is.na(x)==FALSE]))
  n <- length(x)
  G <- sum(x * 1:n)
  G <- 2 * G/sum(x) - (n + 1)
  ifelse(
    isSample,
    return(G / n),
    return(G / (n - 1))
  )
}

# This function calculates the outcome variables as in SG, and outputs a 
# dataframe which contains the ABM's parameters and outcome variables.
SG2013calculateGlobalVariables <- function(a, parameters){
  globals <- parameters
  globals$giniIndex <- giniIndex(a$resources)
  nEvaluationErrors <- nrow(subset(
    a,
    a$role == "author" & a$gotPublished == FALSE & a$deservedPublished == TRUE
  ))
  
  # the following outcome measure is also called "qual-index" in the code by SG.
  globals$evaluationBias <- nEvaluationErrors / sum(a$gotPublished) * 100
  
  # The following is also called "prod-loss" in the code by SG. This way of
  # calculating it imitates their original implementation.
  qBestSubmissions <- sum(subset(
    a, a$role == "author" & a$deservedPublished == TRUE)$submissionQuality)
  qPublishedSubmissions <- sum(subset(
    a, a$role == "author" & a$gotPublished == TRUE)$submissionQuality)
  globals$resourceLoss <- (qBestSubmissions - qPublishedSubmissions) /
    qBestSubmissions * 100
  
  # The following is "rev-prod-index" in the original code. Also follows the way
  # it was originally implemented.
  globals$reviewingExpenses <- 
    sum(a$costOfReviewing / sum(a$costOfAuthoring)) * 100
  
  globals$systemProductivity <- sum(a$resources)
  return(globals)
}